// $Id: djinnid.c,v 1.4 2003/11/11 17:44:58 ensc Exp $    --*- c++ -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "util.h"
#include "djinni.h"
#include "djinniprocess.h"
#include "wrappers.h"
#include "wrappers-io.h"
#include "exithandler.h"
#include "util-io.h"
#include "msg-strings.h"
#include "logging.h"

#include <libgen.h>
#include <getopt.h>
#include <sys/wait.h>

int	wrapper_exit_code = 1;

static struct option const
CMDLINE_OPTIONS[] = {
  { "help",     no_argument,  0, 'h' },
  { "version",  no_argument,  0, 'v' },
  { "daemon",   no_argument,  0, 'd' },
  { 0,0,0,0 }
};

struct ArgInfo
{
    bool		do_fork;
    char const *	pid_file;
    char const *	log_file;
    char const *	err_file;
};

#include "daemonize.hc"

static void
showHelp(int fd, char *cmd, int res)
{
  WRITE_MSG(fd, "Usage:  ");
  WRITE_STR(fd, basename(cmd));
  WRITE_MSG(fd,
	    " [--help] [--version] [--daemon|-d]\n\n"
	    "Please report bugs to " PACKAGE_BUGREPORT "\n");
  exit(res);
}

static void
showVersion()
{
  WRITE_MSG(1,
	    "djinnid " VERSION " -- fulfills wishes when you are rubbing it\n"
	    "This program is part of " PACKAGE_STRING "\n\n"
	    "Copyright (C) 2003 Enrico Scholz\n"
	    VERSION_COPYRIGHT_DISCLAIMER);
  exit(0);
}

static void
setBaseEnv()
{
  Esetenv("LANG", "C", 1);
  Esetenv("DJINNI_LIBDIR", PKGLIBDIR, 1);
}

static void
getArguments(struct ArgInfo *args, int argc, char *argv[])
{
  while (1) {
    int		c = getopt_long(argc, argv, "hvd", CMDLINE_OPTIONS, 0);
    if (c==-1) break;

    switch (c) {
      case 'h'	:  showHelp(1, argv[0], 0);
      case 'v'	:  showVersion();
      case 'd'	:  args->do_fork = true; break;
      default	:
	WRITE_MSG(2, "Try '");
	WRITE_STR(2, argv[0]);
	WRITE_MSG(2, " --help\" for more information.\n");
	exit(1);
	break;
    }
  }
}

int main(int argc, char *argv[])
{
  struct ArgInfo	args = {
    .do_fork  = false,
    .pid_file = DJINNID_PIDFILE,
    .log_file = "/dev/null",
    .err_file = "/dev/null",
  };

  getArguments(&args, argc, argv);

  int			stat_fd;
  if (args.do_fork) stat_fd = daemonize(&args);
  else              stat_fd = -1;

  Logging_newProcess("@main");
  
  ExitHandler_init();
  umask(077);
  if (!getContextOfPid_init()) return EXIT_FAILURE;

  setBaseEnv();

    // To hearbeat-connection between the main-program and the top-level
    // djinni-process. It is required to terminate the djinni-process when
    // main exits (e.g. because of SIGKILL).
  int			heartbeat_fd[2];
  Epipe(heartbeat_fd);
  
  pid_t			pid = Efork();
  if (pid==0) {
    struct DjinniProcess	*djinni;

    Logging_newProcess("@top");

    ExitHandler_clear();
    Eclose(heartbeat_fd[1]);
    djinni = DjinniProcess_new(Eopen("/", O_RDONLY, 0),
			       DJINNI_CFG_DIR,
			       DJINNI_STATE_DIR,
			       0, heartbeat_fd[0]);

    if (stat_fd!=-1) {
      pid_t	pid = getpid();
      EwriteAll(stat_fd, &pid, sizeof pid);
      Eclose(stat_fd);
    }

    exit(DjinniProcess_run(djinni));
  }

  Xclose(stat_fd);
  Eclose(heartbeat_fd[0]);
  getContextOfPid_detach();
  while (wait4(-1,0,0,0)!=-1 || errno!=ECHILD) {}

  if (args.do_fork) unlink(args.pid_file);
  Logging_timestamp(1); WRITE_MSG(1, "exited usually\n");
}
