// $Id: exithandler.c,v 1.1 2003/11/04 22:32:40 ensc Exp $    --*- c -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "exithandler.h"
#include "util-io.h"

#include <stdlib.h>
#include <assert.h>

#define HANDLER_COUNT	32
struct ExitHandler
{
    struct {
	ExitHandlerFunction		fn;
	void *				arg;
    }			function[HANDLER_COUNT];
    size_t		len;
};

static struct ExitHandler	*cur_handler=0;
static struct ExitHandler	handler = {
  .len = 0
};

static void
func()
{
  size_t	i;
  
  assert(cur_handler!=0);
  if (cur_handler==0) return;

  for (i=cur_handler->len; i>0;) {
    --i;
    (*cur_handler->function[i].fn)(cur_handler->function[i].arg);
  }
}

void
ExitHandler_init()
{
  if (cur_handler!=0) {
    WRITE_MSG(2, "Exithandler already initialized\n");
    exit(1);
  }

  cur_handler = &handler;
  atexit(func);
}

bool
ExitHandler_addHandler(ExitHandlerFunction fn, void *arg)
{
  if (handler.len>=HANDLER_COUNT) return false;

  handler.function[handler.len].fn  = fn;
  handler.function[handler.len].arg = arg;
  ++handler.len;

  return true;
}

void
ExitHandler_clear()
{
  handler.len = 0;
}
