// $Id: readcreds.c,v 1.1 2003/11/04 22:32:40 ensc Exp $    --*- c++ -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "util-io.h"
#include "djinni.h"

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/socket.h>

bool
readCreds(int fd, void *buf, size_t buf_len, struct ucred *cred)
{
  struct msghdr		msg;
  struct iovec		iov[1];
  struct cmsghdr        *cmptr;
  size_t		len;
  char                  control[CMSG_SPACE(sizeof(*cred))];

  msg.msg_name       = 0;
  msg.msg_namelen    = 0;
  msg.msg_control    = control;
  msg.msg_controllen = sizeof(control);
  msg.msg_flags      = 0;
  msg.msg_iov        = iov;
  msg.msg_iovlen     = 1;

  iov[0].iov_base = buf;
  iov[0].iov_len  = buf_len;

  do {
    len = recvmsg(fd, &msg, 0);
  } while (len==(size_t)(-1) && (errno==EINTR || errno==EAGAIN));

    // TODO: Logging
  if (len==(size_t)(-1)) {
    perror("recvmsg()");
    return false;
  }

  if (len!=buf_len)                                return false;
  if (msg.msg_controllen < sizeof(struct cmsghdr)) return false;

  for (cmptr = CMSG_FIRSTHDR(&msg); cmptr != NULL;
       cmptr = CMSG_NXTHDR(&msg, cmptr)) {
    if (cmptr->cmsg_len   != sizeof(control) ||
	cmptr->cmsg_level != SOL_SOCKET ||
	cmptr->cmsg_type  != SCM_CREDENTIALS) continue;

    memcpy(cred, CMSG_DATA(cmptr), sizeof(*cred));
    return true;
  }

  return false;
}
